import base64
import logging
import uuid

from cryptography.hazmat.backends import default_backend
from cryptography.x509 import load_der_x509_certificate, NameOID
from esteid_certificates import get_certificate
from oscrypto.asymmetric import Certificate as OsCryptoCertificate

from pyasice.ocsp import OCSP

from .. import settings
from ..authentication import Authenticator
from ..authentication.types import AuthenticationResult
from ..constants import HASH_SHA256
from ..exceptions import ActionInProgress, InvalidIdCode, InvalidParameter, InvalidParameters
from ..types import CertificateHolderInfo
from ..util import generate_hash, secure_random
from .types import LibraryAuthenticateResponse


logger = logging.getLogger(__name__)


class IdCardAuthenticator(Authenticator):
    certificate: bytes

    hash_type: str

    _certificate_handle: "OsCryptoCertificate"

    def setup(self, initial_data: dict = None):
        # no setup required for web-eid based authentication
        pass

        self.hash_type = initial_data.get("algorithm", HASH_SHA256) or HASH_SHA256

    def authenticate(self):
        """This is called by the front-end during the start of the authentication process

        The method generates a challenge nonce and returns it to the front-end to be used in the
        web-eid `authenticate` call.
        """

        random_bytes = secure_random(64)
        hash_value = generate_hash(self.hash_type, random_bytes)
        hash_value_b64 = base64.b64encode(hash_value).decode()

        self.save_session_data(
            session_id=uuid.uuid4().hex,
            hash_value_b64=hash_value_b64,
        )

        raise ActionInProgress(
            data={
                "nonce": hash_value_b64,
            }
        )

    def poll(self, initial_data: dict = None) -> AuthenticationResult:
        hash_value_b64 = self.session_data.hash_value_b64

        if not isinstance(initial_data, dict):
            raise InvalidParameters("Missing required parameters")

        auth_response = LibraryAuthenticateResponse(**initial_data)

        try:
            auth_response.is_valid()
        except (InvalidIdCode, InvalidParameter):
            # Just to be explicit
            raise
        except ValueError as e:
            raise InvalidParameters("Invalid parameters") from e

        certificate = load_der_x509_certificate(auth_response.certificate_bytes, default_backend())

        # This method:
        #
        # - validates the signature matches the certificate and the original nonce
        # - indirectly validates the certificate (by loading it with cryptography)
        auth_response.validate_signature(certificate, self.origin, hash_value_b64.encode())

        # Now we verify the certificate validity via OCSP. This ensures we comply with the following parts of the spec:
        #
        # - validate the certificate
        # - validate the certificate expiry via OCSP
        # - validate that the current time falls within the authentication certificate's validity period
        # - validates that the authentication certificate does not contain any disallowed policies
        # - validates that the authentication certificate is signed by a trusted certificate authority
        # - validates that the purpose of the authentication certificate's key usage is client authentication

        try:
            issuer_common_name = certificate.issuer.get_attributes_for_oid(NameOID.COMMON_NAME)[0].value
        except AttributeError:
            issuer_common_name = certificate.asn1.issuer.native["common_name"]

        issuer_cert = get_certificate(issuer_common_name)

        ocsp = OCSP(settings.OCSP_URL)
        ocsp.validate(auth_response.certificate_bytes, issuer_cert, auth_response.signature_bytes)

        # Finally we construct a holder info object from the certificate, so we can return the user's details in an
        # authentication result
        cert_holder_info = CertificateHolderInfo.from_certificate(auth_response.certificate_bytes)

        return AuthenticationResult(
            country=cert_holder_info.country,
            id_code=cert_holder_info.id_code,
            given_name=cert_holder_info.given_name,
            surname=cert_holder_info.surname,
            certificate_b64=auth_response.unverifiedCertificate,
        )



# pragma: no cover
import logging
import os
import subprocess
import warnings
from tempfile import NamedTemporaryFile

from django.utils.encoding import force_bytes, force_str
from esteid_certificates import get_certificate_file_name, UnknownCertificateError


warnings.warn("This module is deprecated. Please use the new signing API", DeprecationWarning)


logger = logging.getLogger(__name__)


class OCSPVerifier(object):
    def __init__(self, certificate, issuer, ocsp_url, va_file_path):
        self.certificate = certificate
        self.issuer = issuer
        if isinstance(self.issuer, str):
            self.issuer = self.parse_issuer(issuer)

        self.ocsp_url = ocsp_url
        self.va_file_path = va_file_path

    def verify(self):
        if not self.certificate:
            return 1

        if not self.issuer:
            return 2

        issuer_cn = self.issuer.get("CN", None)
        if not issuer_cn:
            return 3

        try:
            issuer_cert_filename = get_certificate_file_name(issuer_cn)
        except UnknownCertificateError:
            return 4

        # Save cert to a temporary file
        with NamedTemporaryFile(delete=False) as cert_file:
            cert_file.write(force_bytes(self.certificate.replace("\t", "")))
            cert_file.close()

            # Run openssl ocsp verify command
            args = [
                "openssl",
                "ocsp",
                f"-issuer {issuer_cert_filename}",
                f"-cert {cert_file.name}",
                f"-url {self.ocsp_url}",
                f"-VAfile {self.va_file_path}",
            ]

            try:
                output = subprocess.check_output(" ".join(args), stderr=subprocess.STDOUT, shell=True)
                output = force_str(output)

                if ": good" in output:
                    result = 0

                elif ": revoked" in output:
                    result = 6

                else:
                    logger.info("openssl ocsp: unknown output: %s", output)
                    result = 7

            except subprocess.CalledProcessError as e:
                logger.error("openssl ocsp: failed with output: %s", e.output)
                result = 5

            # delete temporary file
            os.unlink(cert_file.name)

        return result

    @classmethod
    def parse_issuer(cls, issuer):
        res = {}
        issuer = issuer.strip().strip("/").split("/")

        for part in issuer:
            part = part.split("=")
            res[part[0]] = part[1]

        return res



from django.core.exceptions import ImproperlyConfigured


MOBILE_ID_DEMO_URL = "https://tsp.demo.sk.ee/mid-api"
MOBILE_ID_LIVE_URL = "https://mid.sk.ee/mid-api"

MOBILE_ID_DEMO_SERVICE_NAME = "DEMO"
MOBILE_ID_DEMO_SERVICE_UUID = "00000000-0000-0000-0000-000000000000"

SMART_ID_DEMO_URL = "https://sid.demo.sk.ee/smart-id-rp/v1"
SMART_ID_LIVE_URL = "https://rp-api.smart-id.com/v1"

SMART_ID_DEMO_SERVICE_NAME = MOBILE_ID_DEMO_SERVICE_NAME
SMART_ID_DEMO_SERVICE_UUID = MOBILE_ID_DEMO_SERVICE_UUID

OCSP_DEMO_URL = "http://demo.sk.ee/ocsp"
OCSP_LIVE_URL = "http://ocsp.sk.ee/"

TSA_DEMO_URL = "http://demo.sk.ee/tsa/"
TSA_LIVE_URL = "http://tsa.sk.ee"

# Set of supported hash algorithms. Same for both MobileID and SmartID
# see https://github.com/SK-EID/smart-id-documentation#33-hash-algorithms
# https://github.com/SK-EID/MID#231-supported-hashing-algorithms
HASH_SHA256 = "SHA256"
HASH_SHA384 = "SHA384"
HASH_SHA512 = "SHA512"
HASH_ALGORITHMS = {
    HASH_SHA256,
    HASH_SHA384,
    HASH_SHA512,
}


class Countries:
    """Mainly used by SmartID, also phone code"""

    ESTONIA = "EE"
    LATVIA = "LV"
    LITHUANIA = "LT"

    ALL = (ESTONIA, LATVIA, LITHUANIA)


class Languages:
    """Used by MobileID"""

    ENG = "ENG"
    EST = "EST"
    LIT = "LIT"
    RUS = "RUS"

    ALL = (ENG, EST, LIT, RUS)

    # also allow ISO 639-1 (or alpha-2) codes
    _MAP_ISO_639_1_TO_MID = {
        "en": ENG,
        "et": EST,
        "lt": LIT,
        "ru": RUS,
    }

    @classmethod
    def identify_language(cls, language_code):
        all_langs = {**cls._MAP_ISO_639_1_TO_MID, **{c.lower(): c for c in cls.ALL}}
        try:
            language_code_lower = language_code.lower()
            return all_langs[language_code_lower]
        except (AttributeError, KeyError) as e:
            raise ImproperlyConfigured(f"Language should be one of {','.join(all_langs)}, got `{language_code}`") from e



import logging
from typing import List

from pyasice import Container, XmlSignature

from esteid.exceptions import ActionInProgress, InvalidIdCode, InvalidParameter, InvalidParameters
from esteid.signing import DataFile, Signer

from .i18n import TranslatedMobileIDService
from .types import MobileIdSessionData, UserInput


logger = logging.getLogger(__name__)


class MobileIdSigner(Signer):
    phone_number: str
    id_code: str
    language: str

    SessionData = MobileIdSessionData
    session_data: MobileIdSessionData

    def setup(self, initial_data: dict = None):
        """
        Receives user input via POST: `id_code`, `phone_number`, `language`
        """
        if not isinstance(initial_data, dict):
            raise InvalidParameters("Missing required parameters")

        user_input = UserInput(**initial_data)

        try:
            user_input.is_valid()
        except (InvalidIdCode, InvalidParameter):
            # Just to be explicit
            raise
        except ValueError as e:
            raise InvalidParameters("Invalid parameters") from e

        self.id_code = user_input.id_code
        self.phone_number = user_input.phone_number
        self.language = user_input.language

    def prepare(self, container=None, files: List[DataFile] = None) -> dict:
        container = self.open_container(container, files)

        service = TranslatedMobileIDService.get_instance()

        certificate = service.get_certificate(self.id_code, self.phone_number)

        xml_sig = container.prepare_signature(certificate)

        sign_session = service.sign(self.id_code, self.phone_number, xml_sig.signed_data(), language=self.language)

        self.save_session_data(
            digest=sign_session.digest,
            container=container,
            xml_sig=xml_sig,
            session_id=sign_session.session_id,
        )

        return {
            "verification_code": sign_session.verification_code,
        }

    def finalize(self, data: dict = None) -> Container:
        digest = self.session_data.digest
        session_id = self.session_data.session_id

        temp_signature_file = self.session_data.temp_signature_file
        temp_container_file = self.session_data.temp_container_file

        with open(temp_signature_file, "rb") as f:
            xml_sig = XmlSignature(f.read())

        service = TranslatedMobileIDService.get_instance()

        try:
            status = service.sign_status(session_id, xml_sig.get_certificate_value(), digest)
        except ActionInProgress:
            # Just to be explicit about the InProgress status
            raise

        xml_sig.set_signature_value(status.signature)

        self.finalize_xml_signature(xml_sig)

        container = Container.open(temp_container_file)
        container.add_signature(xml_sig)

        return container

    def save_session_data(self, *, digest: bytes, container: Container, xml_sig: XmlSignature, session_id: str):
        data_obj = self.session_data
        data_obj.session_id = session_id

        super().save_session_data(digest=digest, container=container, xml_sig=xml_sig)



import re

from esteid.constants import Countries
from esteid.exceptions import InvalidIdCode, InvalidParameter


ID_CODE_EE_REGEXP = re.compile(r"^[1-6] \d{2} [01]\d [0123]\d \d{4}$", re.VERBOSE)
ID_CODE_LT_REGEXP = ID_CODE_EE_REGEXP
ID_CODE_LV_REGEXP = re.compile(r"^\d{6}-\d{5}$")


def id_code_ee_is_valid(id_code: str) -> bool:
    """
    Validates Estonian ID code, including checksum.

    https://et.wikipedia.org/wiki/Isikukood
    """
    if isinstance(id_code, str) and bool(re.match(ID_CODE_EE_REGEXP, id_code)):
        step1_factors = "1234567891"
        checksum = sum(int(i) * int(d) for i, d in zip(step1_factors, id_code[:10])) % 11
        if checksum == 10:
            step2_factors = "3456789123"
            checksum = sum(int(i) * int(d) for i, d in zip(step2_factors, id_code[:10])) % 11
            if checksum == 10:
                checksum = 0
        if int(id_code[-1]) == checksum:
            return True
    return False


def id_code_lv_is_valid(id_code: str) -> bool:
    """
    Validates Latvian ID code

    Given the input in the following format ABCDEF-XGHIZ,
    Z must equal to (1101-(1*A+6*B+3*C+7*D+9*E+10*F+5*X+8*G+4*H+2*I)) | Mod 11 | Mod 10.
    """
    if isinstance(id_code, str) and bool(re.match(ID_CODE_LV_REGEXP, id_code)):
        id_code = id_code.replace("-", "")
        factors = [1, 6, 3, 7, 9, 10, 5, 8, 4, 2]
        checksum = (1101 - sum(i * int(d) for i, d in zip(factors, id_code[:10]))) % 11 % 10
        if int(id_code[-1]) == checksum:
            return True
    return False


# Lithuanian ID code format is the same as Estonian.
id_code_lt_is_valid = id_code_ee_is_valid


ID_CODE_VALIDATORS = {
    Countries.ESTONIA: id_code_ee_is_valid,
    Countries.LATVIA: id_code_lv_is_valid,
    Countries.LITHUANIA: id_code_lt_is_valid,
}


def validate_id_code(id_code, country):
    try:
        validator = ID_CODE_VALIDATORS[country]
    except (KeyError, TypeError) as e:
        raise InvalidParameter(f"Unsupported country '{country}'", param="country") from e

    if not validator(id_code):
        # Find country name from Countries attributes
        cty = next(name for name, value in iter(Countries.__dict__.items()) if value == country)
        raise InvalidIdCode(f"ID code '{id_code}' is not valid for {cty.capitalize()}")



import logging
from typing import Union
from uuid import UUID

import requests

from .exceptions import (
    ActionInProgress,
    BadRequest,
    EsteidError,
    InvalidCredentials,
    OfflineError,
    SessionDoesNotExist,
    UnsupportedClientImplementation,
    UpstreamServiceError,
)


logger = logging.getLogger(__name__)


class BaseSKService:
    """Base class for SK Authentication and signing services"""

    NAME: str

    class ProcessingStates:
        # Documentation states that the state can only be RUNNING or COMPLETE
        # see https://github.com/SK-EID/smart-id-documentation#464-response-structure
        # and https://github.com/SK-EID/MID#335-response-structure
        RUNNING = "RUNNING"
        COMPLETE = "COMPLETE"

        ALL = (
            RUNNING,
            COMPLETE,
        )

    _REQUEST_HEADERS = {
        "Content-Type": "application/json",
    }

    def __init__(self, rp_uuid: Union[UUID, str], rp_name: str, api_root: str):
        if not isinstance(rp_uuid, UUID):
            try:
                UUID(rp_uuid)
            except (TypeError, ValueError) as e:
                raise EsteidError("rp_uuid expected to be a valid UUID") from e
        self.rp_uuid = str(rp_uuid)
        self.rp_name = rp_name
        self.api_root = api_root

        self.session = requests.Session()

    def rp_params(self, data: dict):
        return {
            **data,
            "relyingPartyUUID": self.rp_uuid,
            "relyingPartyName": self.rp_name,
        }

    def api_url(self, endpoint):
        return f"{self.api_root}{endpoint}"

    def invoke(self, endpoint, method="GET", query=None, data=None):
        query = query or {}

        if method != "GET":
            data = self.rp_params(data or {})

        req = requests.Request(
            method=method,
            url=self.api_url(endpoint),
            params=query,
            json=data,
            headers=self._REQUEST_HEADERS,
        )
        prepared = req.prepare()

        try:
            # Attempt to fulfill the request
            response = self.session.send(prepared)

            # ensure we don't mask errors
            response.raise_for_status()

        except (requests.ConnectionError, requests.Timeout, ConnectionError) as e:
            logger.exception("Failed to get a response from %s.", req.url)
            raise OfflineError(service=self.NAME) from e

        except requests.HTTPError as e:
            status_code = e.response.status_code
            if status_code == 401:
                logger.exception("API authentication failure at %s: %s", req.url, e.response.text)
                raise InvalidCredentials from e

            if status_code == 400:
                logger.exception("Bad Request to %s. Response:\n%s", req.url, e.response.text)
                raise BadRequest from e

            if status_code == 480:
                logger.exception("Service reported Unsupported Client at %s. Response:\n%s", req.url, e.response.text)
                raise UnsupportedClientImplementation(service=self.NAME) from e

            # 580 System is under maintenance, retry later.
            # see https://github.com/SK-EID/smart-id-documentation#413-http-status-code-usage
            # (Note: Though not documented, Mobile ID also returns this occasionally.)
            if status_code == 580:
                logger.exception("Service %s on maintenance.", self.NAME)
                raise OfflineError("Maintenance", service=self.NAME) from e

            # Raise proxy errors as OfflineError
            if status_code in [502, 503, 504]:
                logger.exception("Service %s not available (HTTP %s).", req.url, status_code)
                raise OfflineError(f"Proxy error: {status_code}", service=self.NAME) from e

            # UpstreamServiceError for everything else. The caller can use `exc.__cause__` to get the original HTTPError
            # Do not log these errors here, because they can be related to user input, depending on the endpoint.
            raise UpstreamServiceError(f"Service {self.NAME} returned an unidentified error", service=self.NAME) from e

        try:
            return response.json()
        except ValueError as e:  # pragma: no cover
            logger.exception("Failed to parse %s response as JSON:\n%s", req.url, response.content)
            raise UpstreamServiceError("Failed to parse response as JSON", service=self.NAME) from e

    def poll_session(self, session_id, endpoint_url, timeout=None) -> dict:
        """
        Polls session status with a specified timeout and returns the received result.

        The timeout defaults to 500ms to avoid extremely long blocking. If it is done asynchronously via Celery,
        it can be increased as per the docs below.

        https://github.com/SK-EID/MID#334-long-polling
        https://github.com/SK-EID/smart-id-documentation#46-session-status
        """
        endpoint_url = endpoint_url.format(session_id=session_id)
        try:
            data = self.invoke(
                endpoint_url,
                query={
                    "timeoutMs": timeout or 500,
                },
            )

        except UpstreamServiceError as e:
            cause = getattr(e, "__cause__", None)
            if isinstance(cause, requests.HTTPError):
                response = cause.response  # pylint: disable=no-member
                if response.status_code == 404:
                    raise SessionDoesNotExist(session_id) from e

                logger.exception(
                    "The %s service returned an error %s at %s. Response:\n %s",
                    self.NAME,
                    response.status_code,
                    endpoint_url,
                    response.text,
                )
            else:
                logger.exception("Unidentified error at %s.", endpoint_url)
            raise

        state = data["state"]
        if state == self.ProcessingStates.RUNNING:
            raise ActionInProgress(session_id)

        if state != self.ProcessingStates.COMPLETE:
            logger.exception("Unrecognized state %s at %s", state, endpoint_url)
            raise UpstreamServiceError(service=self.NAME)

        return data



from django.core.exceptions import ImproperlyConfigured
from django.utils.translation import gettext_lazy as _

from esteid import settings

from .base import MobileIDService


class TranslatedMobileIDService(MobileIDService):
    """MobileIDService w/ translatable messages and django settings-aware"""

    # NOTE:!!!! 20-char limit !!!!!!
    DISPLAY_TEXT_AUTH = _("Authenticate")
    DISPLAY_TEXT_SIGN = _("Sign")

    @classmethod
    def get_instance(cls) -> "TranslatedMobileIDService":
        cls.configuration_valid()

        return cls(
            rp_uuid=settings.MOBILE_ID_SERVICE_UUID,
            rp_name=settings.MOBILE_ID_SERVICE_NAME,
            api_root=settings.MOBILE_ID_API_ROOT,
        )

    @staticmethod
    def configuration_valid():
        if not (settings.MOBILE_ID_SERVICE_NAME and settings.MOBILE_ID_SERVICE_UUID):
            raise ImproperlyConfigured("Both MOBILE_ID_SERVICE_NAME and MOBILE_ID_SERVICE_UUID must be set")



class EndResults:
    """Taken FWIW from the docs"""

    OK = "OK"
    TIMEOUT = "TIMEOUT"
    NOT_MID_CLIENT = "NOT_MID_CLIENT"
    USER_CANCELLED = "USER_CANCELLED"
    SIGNATURE_HASH_MISMATCH = "SIGNATURE_HASH_MISMATCH"
    PHONE_ABSENT = "PHONE_ABSENT"
    DELIVERY_ERROR = "DELIVERY_ERROR"
    SIM_ERROR = "SIM_ERROR"

    ALL = (
        OK,
        TIMEOUT,
        NOT_MID_CLIENT,
        USER_CANCELLED,
        SIGNATURE_HASH_MISMATCH,
        PHONE_ABSENT,
        DELIVERY_ERROR,
        SIM_ERROR,
    )



__all__ = ["SmartIdAuthenticator", "SmartIDError", "SmartIdSigner"]

from ..exceptions import EsteidError as SmartIDError
from .authenticator import SmartIdAuthenticator
from .signer import SmartIdSigner



from typing import List

import pyasice

from esteid.exceptions import InvalidParameter
from esteid.signing import Container, DataFile, Signer


class MySigner(Signer):
    def prepare(self, container: pyasice.Container = None, files: List[DataFile] = None) -> dict:
        container = self.open_container(container, files)
        xml_sig = pyasice.XmlSignature.create()

        self.save_session_data(digest=b"test", container=container, xml_sig=xml_sig)

        return {"verification_code": "1234"}

    def finalize(self, data=None) -> Container:
        return Container.open(self.session_data.temp_container_file)


class MyPostSigner(MySigner):
    """
    Requires POST method parameters to init and finalize
    """

    def setup(self, initial_data: dict = None):
        try:
            initial_data["certificate"]
        except (TypeError, KeyError):
            raise InvalidParameter(param="certificate")

    def finalize(self, data=None) -> Container:
        try:
            data["signature_value"]
        except (TypeError, KeyError):
            raise InvalidParameter(param="signature_value")

        return super().finalize()



# pragma: no cover
import os
import warnings
from io import BytesIO
from typing import List, Union

from pyasice import Container

from esteid.types import DataFile


warnings.warn("This module is deprecated. Please use the new API", DeprecationWarning)


SESSION_KEY = "__esteid_session"


class EsteidSessionError(Exception):
    pass


def get_esteid_session(request):
    return request.session.get(SESSION_KEY, {})


def start_esteid_session(request):
    delete_esteid_session(request)
    return {}


def update_esteid_session(request, **kwargs):
    session = request.session
    data = session.get(SESSION_KEY, {})
    data.update(kwargs)
    session[SESSION_KEY] = data  # Django session object requires setting keys explicitly


def delete_esteid_session(request):
    session_data = request.session.pop(SESSION_KEY, {})

    temp_signature_file = session_data.get("temp_signature_file")
    if temp_signature_file:
        try:
            os.unlink(temp_signature_file)
        except OSError:
            pass

    temp_container_file = session_data.get("temp_container_file")
    if temp_container_file:
        try:
            os.unlink(temp_container_file)
        except OSError:
            pass


def open_container(container_path: Union[str, BytesIO] = None, files_to_sign: List[DataFile] = None):
    """Open an existing BDoc container, or create a container with files to sign"""
    if container_path:
        # Take files from an existing container, ignore files_to_sign
        if isinstance(container_path, str):
            container = Container.open(container_path)
        else:
            container = Container(container_path)
    elif files_to_sign:
        container = Container()
        for data_file in files_to_sign:
            container.add_file(data_file.file_name, data_file.content, data_file.mimetype)
    else:
        raise ValueError("No files to sign")  # this is a 500 error because it's not the user's fault

    return container



import base64
import logging

from esteid.exceptions import ActionInProgress, InvalidIdCode, InvalidParameter, InvalidParameters

from ..authentication import Authenticator
from ..authentication.types import AuthenticationResult
from ..types import CertificateHolderInfo
from .i18n import TranslatedMobileIDService
from .types import UserInput


logger = logging.getLogger(__name__)


class MobileIdAuthenticator(Authenticator):
    phone_number: str
    id_code: str
    language: str

    def setup(self, initial_data: dict = None):
        """
        Receives user input via POST: `id_code`, `phone_number`, `language`
        """
        if not isinstance(initial_data, dict):
            raise InvalidParameters("Missing required parameters")

        user_input = UserInput(**initial_data)

        try:
            user_input.is_valid()
        except (InvalidIdCode, InvalidParameter):
            # Just to be explicit
            raise
        except ValueError as e:
            raise InvalidParameters("Invalid parameters") from e

        self.id_code = user_input.id_code
        self.phone_number = user_input.phone_number
        self.language = user_input.language

    def authenticate(self):
        service = TranslatedMobileIDService.get_instance()

        auth_initial_result = service.authenticate(self.id_code, self.phone_number, language=self.language)

        self.save_session_data(
            session_id=auth_initial_result.session_id, hash_value_b64=auth_initial_result.hash_value_b64
        )

        raise ActionInProgress(
            data={
                "verification_code": auth_initial_result.verification_code,
            }
        )

    def poll(self, initial_data: dict = None) -> AuthenticationResult:
        session_id = self.session_data.session_id
        hash_value_b64 = self.session_data.hash_value_b64

        service = TranslatedMobileIDService.get_instance()

        # raises ActionInProgress if not received a final result
        auth_result = service.status(session_id, hash_value=base64.b64decode(hash_value_b64))

        cert_holder_info = CertificateHolderInfo.from_certificate(auth_result.certificate)

        return AuthenticationResult(
            country=cert_holder_info.country,
            id_code=cert_holder_info.id_code,
            given_name=cert_holder_info.given_name,
            surname=cert_holder_info.surname,
            certificate_b64=auth_result.certificate_b64,
        )



from ..idcard.authenticator import IdCardAuthenticator


try:
    from django.urls import re_path
except ImportError:  # noqa
    from django.conf.urls import url as re_path

from esteid.mobileid import MobileIdAuthenticator  # noqa
from esteid.signing import Signer
from esteid.smartid import SmartIdAuthenticator  # noqa

from .views import AuthTestRestView, AuthTestView, SigningTestRestView, SigningTestView


# Signing

urlpatterns = [
    re_path(f"^sign/{method}/", SigningTestView.as_view(signing_method=method), name=f"sign-{method}")
    for method in Signer.SIGNING_METHODS
]

urlpatterns += [
    re_path(f"^sign-rest/{method}/", SigningTestRestView.as_view(signing_method=method), name=f"sign-rest-{method}")
    for method in Signer.SIGNING_METHODS
]

# Authentication

urlpatterns += [
    re_path(
        f"^authenticate/{auth_class.get_method_name()}/",
        AuthTestView.as_view(authenticator=auth_class),
        name=f"auth-{auth_class.get_method_name()}",
    )
    for auth_class in [IdCardAuthenticator, MobileIdAuthenticator, SmartIdAuthenticator]
]

urlpatterns += [
    re_path(
        f"^authenticate-rest/{auth_class.get_method_name()}/",
        AuthTestRestView.as_view(authenticator=auth_class),
        name=f"auth-rest-{auth_class.get_method_name()}",
    )
    for auth_class in [IdCardAuthenticator, MobileIdAuthenticator, SmartIdAuthenticator]
]
